import React, { Component, useState, useCallback, createContext, useContext, useMemo, useRef, } from "react";
class ReactUseErrorBoundaryWrappedError extends Error {
    originalError;
    constructor(error) {
        console.warn("react-use-error-boundary: A value was thrown that is not an instance of Error. Thrown values should be instantiated with JavaScript's Error constructor.");
        try {
            super(error);
        }
        catch {
            super("react-use-error-boundary: Could not instantiate an Error with the thrown value. The thrown value can be accessed via the 'originalError' property");
        }
        this.name = "ReactUseErrorBoundaryWrappedError";
        Error.captureStackTrace?.(this, ReactUseErrorBoundaryWrappedError);
        this.originalError = error;
    }
}
class ErrorBoundary extends Component {
    displayName = "ReactUseErrorBoundary";
    componentDidCatch(...args) {
        this.setState({});
        this.props.onError(...args);
    }
    render() {
        return this.props.children;
    }
}
const noop = () => false;
const errorBoundaryContext = createContext({
    componentDidCatch: { current: undefined },
    error: undefined,
    setError: noop,
});
export function ErrorBoundaryContext({ children, }) {
    const [error, setError] = useState();
    const componentDidCatch = useRef();
    const ctx = useMemo(() => ({
        componentDidCatch,
        error,
        setError,
    }), [error]);
    return (React.createElement(errorBoundaryContext.Provider, { value: ctx },
        React.createElement(ErrorBoundary, { error: error, onError: (error, errorInfo) => {
                if (!(error instanceof Error)) {
                    error = new ReactUseErrorBoundaryWrappedError(error);
                }
                setError(error);
                componentDidCatch.current?.(error, errorInfo);
            } }, children)));
}
ErrorBoundaryContext.displayName = "ReactUseErrorBoundaryContext";
export function withErrorBoundary(WrappedComponent) {
    return (props) => (React.createElement(ErrorBoundaryContext, null,
        React.createElement(WrappedComponent, { key: "WrappedComponent", ...props })));
}
export function useErrorBoundary(componentDidCatch) {
    const ctx = useContext(errorBoundaryContext);
    ctx.componentDidCatch.current = componentDidCatch;
    const resetError = useCallback(() => {
        ctx.setError(undefined);
    }, []);
    return [ctx.error, resetError];
}
